//
//  LocationManager.h
//
//  Copyright (c) 2012 ZENRIN DataCom CO., LTD. All Rights Reserved.
//

#import <Foundation/Foundation.h>

#define LOCATION_MANAGER_LIBRARY_VERSION		@"4.0.0"
/*!
 @header LocationManager.h
 
 @brief This is the header file where my super-code is contained.
 
 This file contains the most important method and properties decalaration.
 
 @author ZDC
 @copyright  2017 ZENRIN DataCom CO., LTD. All Rights Reserved.
 @version    4.0.0
 */

/*!
 @brief Enum execution result of measuringStart
 */
typedef NSInteger MeasuringStartResult;
enum {
    /// Failed to start
    MeasuringStartResult_Error              = -1,
    /// Start positioning with standard positioning method
    MeasuringStartResult_Standard           = 1,
    /// Start positioning with a drastically changed positioning method
    MeasuringStartResult_SignificantChange,
    /// Region+Visit operation started with positioning method using
    MeasuringStartResult_RegionVisit,
};


/*!
 @brief Enum positioning result
 */
typedef NSUInteger LocationManagerStatus;
enum {
    /// Positioning successful
    LocationManagerStatusSuccess    = 0,
    /// Positioning error
    LocationManagerStatusError = 2
};

/*!
 @brief Enum measuring method
 */
typedef NSUInteger MeasuringType;
enum {
    /// Standard position information service positioning
	MeasuringType_Standard = 0,
    /// Significantly change location information service measurement
	MeasuringType_Significant
};

/*!
 @brief Enum measurement method change reason
 */
typedef NSUInteger MeasuringTypeChangeReason;
enum {
    /// Measuring type change by battery
	MeasuringTypeChange_Battery = 0,
};

/*!
 @brief Enum authentication environment
 */
typedef NSUInteger AuthEnv;
enum {
    /// production environment
    AuthEnv_Pro = 0,
    /// staging environment
    AuthEnv_Stg,
    /// test environment
    AuthEnv_Test,
};

/*!
 @brief Enum NSError code in measurement callback
 */
typedef NSUInteger ErrCode;
enum {
    /// トークン発行認証失敗
    ErrCode_GetTokenAuthFailed = 1,
    /// 認証不許可
    ErrCode_AuthNotAuthorized,
    /// 測位エラー
    ErrCode_MeasuringError,
    /// バリデーションエラー
    ErrCode_ValidationError,
};

/*!
 @brief Location information class
 */
@interface LocationInfo : NSObject

/*! @brief Location time stamp */
@property (nonatomic, readonly) NSDate* timeStamp;
/*! @brief Location date (YYYYMMDD) */
@property (nonatomic, readonly) NSString* date;
/*! @brief Location time (HHMMSS) */
@property (nonatomic, readonly) NSString* time;
/*! @brief Location latitude */
@property (nonatomic, readonly) double latitude;
/*! @brief Location longitude */
@property (nonatomic, readonly) double longitude;
/*! @brief Location horizontal accuracy */
@property (nonatomic, readonly) double horizontalAccuracy;
/*! @brief Location vertical accuracy */
@property (nonatomic, readonly) double verticalAccuracy;
/*! @brief Location altitude */
@property (nonatomic, readonly) double altitude;
/*! @brief Location speed */
@property (nonatomic, readonly) double speed;
/*! @brief Location bearing */
@property (nonatomic, readonly) double bearing;

@end


/*! @brief Measuring API key (NSString) */
extern NSString * const MeasuringSettingApiKey;
/*! @brief Measuring clientID key (NSString) */
extern NSString * const MeasuringSettingClientIDKey;
/*! @brief Measuring accuracy key (NSNumber<float>) */
extern NSString * const MeasuringSettingAccuracyKey;
/*! @brief Measuring distance key (NSNumber<float>) */
extern NSString * const MeasuringSettingDistanceKey;
/*! @brief Measuring battery level change (NSNumber<float>) */
extern NSString * const MeasuringSettingBatteryLevIntervalKey;
/*! @brief Measuring significant change accuracy key (NSNumber<float>) */
extern NSString * const MeasuringSettingSignificantChangeAccuracyKey;
/*! @brief Measuring secret key (NSString) */
extern NSString * const MeasuringSettingSecretKey;
/*! @brief Measuring authEnv key (NSNumber) */
extern NSString * const MeasuringSettingAuthEnvKey;
/*! @brief Measuring radius of monitoring area (m)(Small) (NSNumber<double>) */
extern NSString * const MeasuringSettingMonitoringAreaSmallRadiusKey;
/*! @brief Measuring radius of monitoring area (m)(Medium) (NSNumber<double>) */
extern NSString * const MeasuringSettingMonitoringAreaMiddleRadiusKey;
/*! @brief Measuring radius of monitoring area (m)(Large) (NSNumber<double>) */
extern NSString * const MeasuringSettingMonitoringAreaLargeRadiusKey;
/*! @brief Measuring radius of monitoring area (m)(Oversize) (NSNumber<double>) */
extern NSString * const MeasuringSettingMonitoringAreaOversizeRadiusKey;
/*! @brief Measuring travel speed (km/h)(Slow) (NSNumber<double>) */
extern NSString * const MeasuringSettingMonitoringSlowSpeedKey;
/*! @brief Measuring travel speed (km/h)(Middle) (NSNumber<double>) */
extern NSString * const MeasuringSettingMonitoringMiddleSpeedKey;
/*! @brief Measuring travel speed (km/h)(High) (NSNumber<double>) */
extern NSString * const MeasuringSettingMonitoringHighSpeedKey;
/*! @brief Measuring low energy mode (NSNumber<BOOL>) */
extern NSString * const MeasuringSettingLowEnergyModeKey;
/*!
 @brief Location Manager Delegate
 */
@protocol LocationManagerDelegate <NSObject>

@required
/*!
 @brief Callback function to notify positioning information

 @param locationInfo LocationInfo
 @param status       LocationManagerStatus
 @param error        NSError
 */
- (void)measuringResult:(LocationInfo *)locationInfo status:(LocationManagerStatus)status error:(NSError *)error;

@optional
/*!
 @brief Callback function to notify the positioning method change

 @param measureStatus MeasuringType
 @param reason        MeasuringTypeChangeReason
 */
- (void)changeMeasureStatus:(MeasuringType)measureStatus reason:(MeasuringTypeChangeReason)reason;

@end


/*! 
 @brief Location Manager
 */
@interface LocationManager : NSObject

@property (nonatomic) id<LocationManagerDelegate> delegate;

/*!
 @brief LocationManager Instance

 @return LocationManager
 */
+ (LocationManager *)defaultLocationManager;

/*!
 @brief Start measuring position
 
 @param settings NSDictionary
 
 @return MeasuringStartResult
 */
- (MeasuringStartResult)measuringStart:(NSDictionary *)settings error:(NSError **)error;

/*!
 @brief Stop measuring position
 */
- (void)measuringStop;

/*!
 @brief Get library version

 @return NSString
 */
- (NSString *)libraryVersion;

/*!
 @brief Value setting library

 @param isOperationValue YES/NO

 @return NSDictionary
 */
- (NSDictionary *)librarySettings:(BOOL)isOperationValue;
/*!
 @brief Get current location

 @param accuracy double
 @param apiKey NSString
 @param clientId NSString
 @param secret NSString
 @param error    NSError

 @return LocationInfo
 */
- (LocationInfo *)currentLocation:(double)accuracy apiKey:(NSString *)apiKey clientId:(NSString*)clientId secret:(NSString*)secret error:(NSError **)error NS_SWIFT_NOTHROW;

/*!
 @brief Get current location

 @param accuracy double
 @param apiKey NSString
 @param clientId NSString
 @param secret NSString
 @param authEnv NSInteger
 @param error    NSError

 @return LocationInfo
 */
- (LocationInfo *)currentLocation:(double)accuracy apiKey:(NSString *)apiKey clientId:(NSString*)clientId secret:(NSString*)secret authEnv:(NSInteger)authEnv error:(NSError **)error NS_SWIFT_NOTHROW;

@end
